//+------------------------------------------------------------------+
//|                                                        Bands.mq4 |
//|                   Copyright 2005-2014, MetaQuotes Software Corp. |
//|                                              http://www.mql4.com |
//+------------------------------------------------------------------+
#property copyright   "2005-2014, MetaQuotes Software Corp."
#property link        "http://www.mql4.com"
#property description "Bollinger Bands"
#property strict

#include <MovingAverages.mqh>
#include <Probability.mqh>

#property indicator_chart_window
#property indicator_buffers 3
#property indicator_color1 LightSeaGreen
#property indicator_color2 LightSeaGreen
#property indicator_color3 LightSeaGreen
//--- indicator parameters
input int    InpBandsPeriod=20;      // Bands Period
input int    InpBandsShift=0;        // Bands Shift
input double InpBandsDeviations=2.0; // Bands Deviations
//+------------------------------------------------------------------+
//| Custom indicator initialization function                         |
//+----------------------------------------------------------------
extern string ExpDate = "2018.02.06";
extern double DaysInYear=365;
extern double DaysInMonth=30;
extern double RFreeRate = 1.15;
extern string CurrencySymbol = "0"; //
//--- buffers
double ExtUpperBuffer[];
double ExtLowerBuffer[];
double ExtStdDevBuffer[];
double ExtCallBuffer[];
double ExtPutBuffer[];
double ExtDeFirstBuffer[];
double ExtMovingBuffer[];

int OnInit(void)
  {
//--- 1 additional buffer used for counting.
   IndicatorBuffers(4);
   IndicatorDigits(Digits);
//--- middle line
   SetIndexStyle(0,DRAW_LINE);
   SetIndexBuffer(0,ExtMovingBuffer);
   SetIndexShift(0,InpBandsShift);
   SetIndexLabel(0,"Bands SMA");
//--- upper band
   SetIndexStyle(1,DRAW_LINE);
   SetIndexBuffer(1,ExtUpperBuffer);
   SetIndexShift(1,InpBandsShift);
   SetIndexLabel(1,"Bands Upper");
//--- lower band
   SetIndexStyle(2,DRAW_LINE);
   SetIndexBuffer(2,ExtLowerBuffer);
   SetIndexShift(2,InpBandsShift);
   SetIndexLabel(2,"Bands Lower");
//--- work buffer
   SetIndexBuffer(3,ExtStdDevBuffer);
   SetIndexBuffer(4,ExtCallBuffer);
   SetIndexBuffer(5,ExtPutBuffer);
   SetIndexBuffer(6,ExtDeFirstBuffer);
//--- check for input parameter
   if(InpBandsPeriod<=0)
     {
      Print("Wrong input parameter Bands Period=",InpBandsPeriod);
      return(INIT_FAILED);
     }
//---
   SetIndexDrawBegin(0,InpBandsPeriod+InpBandsShift);
   SetIndexDrawBegin(1,InpBandsPeriod+InpBandsShift);
   SetIndexDrawBegin(2,InpBandsPeriod+InpBandsShift);
//--- initialization done
   return(INIT_SUCCEEDED);
  }
//+------------------------------------------------------------------+
//| Bollinger Bands                                                  |
//+------------------------------------------------------------------+
int OnCalculate(const int rates_total,
                const int prev_calculated,
                const datetime &time[],
                const double &open[],
                const double &high[],
                const double &low[],
                const double &close[],
                const long &tick_volume[],
                const long &volume[],
                const int &spread[])
  {
   int i,pos;
//---
   if(rates_total<=InpBandsPeriod || InpBandsPeriod<=0)
      return(0);
//--- counting from 0 to rates_total
   ArraySetAsSeries(ExtMovingBuffer,false);
   ArraySetAsSeries(ExtUpperBuffer,false);
   ArraySetAsSeries(ExtLowerBuffer,false);
   ArraySetAsSeries(ExtStdDevBuffer,false);
   ArraySetAsSeries(ExtCallBuffer,false);
   ArraySetAsSeries(ExtPutBuffer,false);
   ArraySetAsSeries(ExtDeFirstBuffer,false);
   ArraySetAsSeries(close,false);
//--- initial zero
   if(prev_calculated<1)
     {
      for(i=0; i<InpBandsPeriod; i++)
        {
         ExtMovingBuffer[i]=EMPTY_VALUE;
         ExtUpperBuffer[i]=EMPTY_VALUE;
         ExtLowerBuffer[i]=EMPTY_VALUE;
        }
     }
//--- starting calculation
   if(prev_calculated>1)
      pos=prev_calculated-1;
   else
      pos=0;
//--- main cycle
   for(i=pos; i<rates_total && !IsStopped(); i++)
     {
      //--- middle line
      ExtMovingBuffer[i]=SimpleMA(i,InpBandsPeriod,close);     
      //--- calculate and write down StdDev
      ExtStdDevBuffer[i]=StdDev_Func(i,close,ExtMovingBuffer,24*InpBandsPeriod);
      ExtDeFirstBuffer[i]= DF(ExtMovingBuffer[i],Strike(ExtMovingBuffer[i]), ExtStdDevBuffer[i]); 
      ExtCallBuffer[i]=ExtMovingBuffer[i]* ndtr(ExtDeFirstBuffer[i])- Strike(ExtMovingBuffer[i])*MathExp(-RFreeRate/100*DTE())*ndtr(ExtDeFirstBuffer[i]-ExtStdDevBuffer[i]*MathSqrt(DTE()));
      ExtPutBuffer[i]=Strike(ExtMovingBuffer[i])*MathExp(-RFreeRate/100*DTE())*ndtr(ExtStdDevBuffer[i]*MathSqrt(DTE())- ExtDeFirstBuffer[i])-ndtr(-ExtDeFirstBuffer[i])*ExtMovingBuffer[i] ;
        
      //--- upper line
      //call= ExtMovingBuffer[i]* ndtr(distrib)- Strike(ExtMovingBuffer[i])*MathExp(-RFreeRate/100*DTE()/365)*ndtr(distrib-ExtStdDevBuffer[i]*MathSqrt(DTE()/365));
      //put= Strike(ExtMovingBuffer[i])*MathExp(-RFreeRate/100*DTE()/365)*ndtr(ExtStdDevBuffer[i]*MathSqrt(DTE()/365)-distrib)-ndtr(-distrib)*ExtMovingBuffer[i] ;
      //ExtUpperBuffer[i]=Strike(ExtMovingBuffer[i])+InpBandsDeviations*ExtStdDevBuffer[i];
      //ExtUpperBuffer[i]=ExtMovingBuffer[i]+InpBandsDeviations*ExtStdDevBuffer[i];
      ExtUpperBuffer[i]=ExtMovingBuffer[i]+ExtCallBuffer[i];
      //lower line
      //ExtLowerBuffer[i]=Strike(ExtMovingBuffer[i])-InpBandsDeviations*ExtStdDevBuffer[i]; 
      //ExtLowerBuffer[i]=ExtMovingBuffer[i]-InpBandsDeviations*ExtStdDevBuffer[i];
      ExtUpperBuffer[i]=ExtMovingBuffer[i]-ExtPutBuffer[i];
      //---
     }
//---- OnCalculate done. Return new prev_calculated.
   return(rates_total);
  }
//+------------------------------------------------------------------+
//| Calculate Standard Deviation                                     |
//+------------------------------------------------------------------+
double StdDev_Func(int position,const double &price[],const double &MAprice[],int period)
  { 
//--- variables
   double StdDev_y_prc, StdDev_mo_prc;
   double StdDev_dTmp=0.0;
//--- check for position
   if(position>=period)
     {
      //--- calcualte StdDev
      for(int i=0; i<period; i++)
         StdDev_dTmp+=MathPow(price[position-i]-MAprice[position],2);
      StdDev_dTmp=MathSqrt(StdDev_dTmp/period);
     }
      StdDev_mo_prc=StdDev_dTmp/sqrt(1/24*1/DaysInMonth)/fabs(price[0]-price[position]);
      //StdDev_pts=StdDev_dTmp/(sqrt(1/24)*sqrt(1/DaysInYear))*sqrt(1/12);
      StdDev_y_prc=StdDev_mo_prc/sqrt(1/12);
//--- return calculated value
     return(StdDev_y_prc);
  }
double Strike (double x)
    {
     string currency;
     if (CurrencySymbol=="0") currency=ChartSymbol(0);
     if (currency == "USDCAD" || currency == "usdcad" || currency == "USDCHF" || currency == "usdchf" ) return 1/(round(1/x*200)/200);
         else if (currency == "GOLD" || currency == "XAUUSD" || currency == "gold" || currency == "xauusd") return round(0.2*x)/0.2;
              else if (currency == "OIL" || currency == "USOIL" || currency == "oil" || currency == "usoil" || currency == "BRENT" || currency == "brent") return round(2*x)/2;
                   else if (currency == "USDJPY" || currency == "usdjpy") return 1/(round(1/x*20000)/20000);
                        else return round(200*x)/200;  
    }  
double DF (double spot, double strk, double s)
    {
      return (log(spot/strk)+(RFreeRate/100+s*s/2)*DTE())/(s*sqrt(DTE()));
    }  
double DTE ()      
    {
      double d;   
      if (((StrToTime(ExpDate)-TimeCurrent())/86400)/DaysInYear>0) d=((StrToTime(ExpDate)-TimeCurrent())/86400)/DaysInYear;
      else d=MathPow(10,-10);
      return d;
    }                            
//+------------------------------------------------------------------+
